import { __decorate, __metadata, __param, __extends } from 'tslib';
import { HostBinding, Input, Component, ViewEncapsulation, Optional, Directive, Inject, ViewContainerRef, ComponentFactoryResolver, Type, NgModule } from '@angular/core';
import { AbstractControl, FormControlName, FormGroupName, FormControl } from '@angular/forms';
import { I18NEXT_NAMESPACE, I18NextModule } from 'angular-i18next';

var ValidationMessage = /** @class */ (function () {
    function ValidationMessage(key, params) {
        if (key === void 0) { key = ''; }
        if (params === void 0) { params = null; }
        this.key = '';
        this.key = key;
        this.params = params;
    }
    return ValidationMessage;
}());

var ValidationMessageComponent = /** @class */ (function () {
    function ValidationMessageComponent(formControlName, formGroupName) {
        this.formControlName = formControlName;
        this.for = null;
        this.acd = formControlName || formGroupName;
        if (this.acd != null)
            this.for = this.acd.control;
    }
    Object.defineProperty(ValidationMessageComponent.prototype, "standalone", {
        get: function () {
            return this.acd == null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageComponent.prototype, "messages", {
        get: function () {
            return this.getValidationMessages();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageComponent.prototype, "firstMessage", {
        get: function () {
            return this.messages[0] ? this.messages[0] : new ValidationMessage();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValidationMessageComponent.prototype, "controlPath", {
        get: function () {
            if (this.acd == null)
                return '';
            return this.acd.path.join('.');
        },
        enumerable: true,
        configurable: true
    });
    ValidationMessageComponent.prototype.getValidationMessages = function () {
        var _this = this;
        if (this.for == null || this.for.errors == null)
            return [];
        var messages = [];
        var keys = Object.keys(this.for.errors);
        keys.forEach(function (k) {
            var errValue = _this.for.errors[k];
            var params = null;
            if (errValue instanceof Object)
                params = errValue;
            else if (errValue !== true) {
                params = {};
                params[k] = errValue;
            }
            messages.push(new ValidationMessage(k, params));
        });
        return messages;
    };
    __decorate([
        HostBinding('class.standalone'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [])
    ], ValidationMessageComponent.prototype, "standalone", null);
    __decorate([
        Input(),
        __metadata("design:type", AbstractControl)
    ], ValidationMessageComponent.prototype, "for", void 0);
    ValidationMessageComponent = __decorate([
        Component({
            selector: 'validation-message',
            template: "\n      <div class=\"error-container\">{{ firstMessage.key + (firstMessage.params ? '(' + firstMessage.params + ')' : '') }}</div>\n    ",
            encapsulation: ViewEncapsulation.None,
            styles: ["\n      validation-message {\n        display: none;\n        width: 100%;\n      }\n      validation-message.standalone,\n      .ng-dirty.ng-invalid + validation-message {\n        display: block;\n      }\n    "]
        }),
        __param(0, Optional()), __param(1, Optional()),
        __metadata("design:paramtypes", [FormControlName, FormGroupName])
    ], ValidationMessageComponent);
    return ValidationMessageComponent;
}());

var ValidationMessageDirective = /** @class */ (function () {
    function ValidationMessageDirective(viewContainer, componentFactoryResolver, vmComp) {
        this.viewContainer = viewContainer;
        this.componentFactoryResolver = componentFactoryResolver;
        this.vmComp = vmComp;
        this.validationMessageComponent = null;
    }
    ValidationMessageDirective.prototype.ngAfterViewInit = function () {
        var factory = this.componentFactoryResolver.resolveComponentFactory(this.vmComp);
        this.ngOnDestroy();
        this.validationMessageComponent = this.viewContainer.createComponent(factory, null, this.viewContainer.injector);
        this.validationMessageComponent.changeDetectorRef.detectChanges();
    };
    ValidationMessageDirective.prototype.ngOnDestroy = function () {
        if (this.validationMessageComponent) {
            this.validationMessageComponent.changeDetectorRef.detach();
        }
    };
    ValidationMessageDirective = __decorate([
        Directive({
            selector: '[validationMessage]'
        }),
        __param(2, Inject(ValidationMessageComponent)),
        __metadata("design:paramtypes", [ViewContainerRef,
            ComponentFactoryResolver,
            Type])
    ], ValidationMessageDirective);
    return ValidationMessageDirective;
}());

var ValidationError = /** @class */ (function () {
    function ValidationError() {
    }
    Object.defineProperty(ValidationError.prototype, "error", {
        get: function () {
            return this.control.getError(this.errorKey);
        },
        enumerable: true,
        configurable: true
    });
    return ValidationError;
}());

var ValidationErrorBuilder = /** @class */ (function () {
    function ValidationErrorBuilder() {
    }
    ValidationErrorBuilder.prototype.build = function (form, prevKey) {
        var validationErrors = [];
        var formControls = form.controls;
        for (var key in formControls) {
            if (formControls.hasOwnProperty(key)) {
                var fullKey = prevKey ? [prevKey, key].join('.') : key;
                var ctrl = formControls[key];
                if (ctrl instanceof FormControl) {
                    if (!ctrl.valid) {
                        for (var erKey in ctrl.errors) {
                            if (ctrl.errors.hasOwnProperty(erKey)) {
                                var valError = new ValidationError();
                                valError.control = ctrl;
                                valError.controlKey = fullKey;
                                valError.errorKey = erKey;
                                validationErrors.push(valError);
                            }
                        }
                    }
                }
                else {
                    var group = ctrl;
                    validationErrors = validationErrors.concat(this.build(group, fullKey));
                }
            }
        }
        return validationErrors;
    };
    return ValidationErrorBuilder;
}());

var declarations = [
    ValidationMessageDirective,
    ValidationMessageComponent,
];
var ɵ0 = ValidationMessageComponent;
var ValidationMessageModule = /** @class */ (function () {
    function ValidationMessageModule() {
    }
    ValidationMessageModule = __decorate([
        NgModule({
            declarations: declarations,
            exports: declarations,
            entryComponents: [
                ValidationMessageComponent
            ],
            providers: [
                ValidationErrorBuilder,
                {
                    provide: ValidationMessageComponent,
                    useValue: ɵ0
                }
            ]
        })
    ], ValidationMessageModule);
    return ValidationMessageModule;
}());

var I18NextValidationMessageComponent = /** @class */ (function (_super) {
    __extends(I18NextValidationMessageComponent, _super);
    function I18NextValidationMessageComponent(i18nextNamespace, formControlName, formGroupName) {
        var _this = _super.call(this, formControlName, formGroupName) || this;
        _this.i18nextNamespace = i18nextNamespace;
        _this.formControlName = formControlName;
        _this.formGroupName = formGroupName;
        _this.VALIDATION = 'validation';
        return _this;
    }
    Object.defineProperty(I18NextValidationMessageComponent.prototype, "i18nextKey", {
        get: function () {
            if (!this.firstMessage.key)
                return '';
            var specificKey = [this.VALIDATION, ['control_specific', this.controlPath, this.firstMessage.key].join('.')].join(':');
            var commonKey = [this.VALIDATION, this.firstMessage.key].join(':');
            var i18nextKeys = [];
            if (this.i18nextNamespace && this.i18nextNamespace !== this.VALIDATION) {
                i18nextKeys.push([this.i18nextNamespace, specificKey].join('.'));
                i18nextKeys.push([this.i18nextNamespace, commonKey].join('.'));
            }
            i18nextKeys.push(specificKey);
            i18nextKeys.push(commonKey);
            return i18nextKeys;
        },
        enumerable: true,
        configurable: true
    });
    I18NextValidationMessageComponent = __decorate([
        Component({
            selector: 'i18next-validation-message',
            template: "\n      <div class=\"error-container\">{{ i18nextKey | i18nextCap: firstMessage.params }}</div>\n    ",
            encapsulation: ViewEncapsulation.None,
            styles: ["\n      i18next-validation-message {\n        display: none;\n        width: 100%;\n        position: relative;\n      }\n      i18next-validation-message.standalone,\n      .ng-dirty.ng-invalid + i18next-validation-message {\n        display: block;\n      }\n    "]
        }),
        __param(0, Inject(I18NEXT_NAMESPACE)),
        __param(1, Optional()),
        __param(2, Optional()),
        __metadata("design:paramtypes", [Object, FormControlName,
            FormGroupName])
    ], I18NextValidationMessageComponent);
    return I18NextValidationMessageComponent;
}(ValidationMessageComponent));

var declarations$1 = [
    I18NextValidationMessageComponent,
];
var ɵ0$1 = I18NextValidationMessageComponent;
var providers = [
    {
        provide: ValidationMessageComponent,
        useValue: ɵ0$1
    }
];
var I18NextValidationMessageModule = /** @class */ (function () {
    function I18NextValidationMessageModule() {
    }
    I18NextValidationMessageModule = __decorate([
        NgModule({
            declarations: declarations$1,
            entryComponents: declarations$1,
            exports: [
                declarations$1,
                ValidationMessageModule
            ],
            imports: [
                I18NextModule,
                ValidationMessageModule
            ],
            providers: providers
        })
    ], I18NextValidationMessageModule);
    return I18NextValidationMessageModule;
}());

export { I18NextValidationMessageModule, ValidationError, ValidationErrorBuilder, ValidationMessage, ValidationMessageComponent, ValidationMessageDirective, ValidationMessageModule, declarations, providers, ɵ0 };
//# sourceMappingURL=angular-validation-message.js.map
